'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _PageHandler = require('./PageHandler');

var _PageHandler2 = _interopRequireDefault(_PageHandler);

var _SerialBatch = require('../../execution/SerialBatch');

var _SerialBatch2 = _interopRequireDefault(_SerialBatch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class PageHandlerRegistry extends _PageHandler2.default {
  /**
   * Creates an instance of HandlerRegistry and creates {@code SerialBatch}
   * instance for pre-handlers and post-handlers.
   *
   * @param {...PageHandler} pageHandlers
   * @memberof HandlerRegistry
   */
  constructor(...pageHandlers) {
    super(...pageHandlers);

    /**
     * Page handlers.
     *
     * @protected
     * @type {PageHandler[]}
     */
    this._pageHandlers = pageHandlers;

    /**
     * Page handlers.
     *
     * @protected
     * @type {Execution}
     */
    this._preManageHandlers = pageHandlers;

    /**
     * Page handlers.
     *
     * @protected
     * @type {Execution}
     */
    this._postManageHandlers = pageHandlers;
  }

  /**
   * @inheritdoc
   */
  init() {
    this._preManageHandlers = Reflect.construct(PageHandlerRegistry.ExecutionMethod, [this._pageHandlers.map(handler => handler.handlePreManagedState.bind(handler))]);

    this._postManageHandlers = Reflect.construct(PageHandlerRegistry.ExecutionMethod, [this._pageHandlers.map(handler => handler.handlePostManagedState.bind(handler))]);
  }

  /**
   * Executes the pre-manage handlers with given arguments
   *
   * @param {?ManagedPage} managedPage
   * @param {?ManagedPage} nextManagedPage
   * @param {{ type: string, event: Event, url: string }} action
   * @return {Promise<any>}
   */
  handlePreManagedState(managedPage, nextManagedPage, action) {
    return this._preManageHandlers.execute(managedPage, nextManagedPage, action);
  }

  /**
   * Executes the post-manage handlers with given arguments
   *
   * @param {?ManagedPage} managedPage
   * @param {?ManagedPage} previousManagedPage
   * @param {{ type: string, event: Event, url: string }} action
   * @return {Promise<any>}
   */
  handlePostManagedState(managedPage, previousManagedPage, action) {
    return this._postManageHandlers.execute(managedPage, previousManagedPage, action);
  }

  /**
   * @inheritdoc
   */
  destroy() {
    this._preManageHandlers = null;
    this._postManageHandlers = null;
    this._pageHandlers = null;
  }
}

exports.default = PageHandlerRegistry;
PageHandlerRegistry.ExecutionMethod = _SerialBatch2.default;

typeof $IMA !== 'undefined' && $IMA !== null && $IMA.Loader && $IMA.Loader.register('ima/page/handler/PageHandlerRegistry', [], function (_export, _context) {
 'use strict';
 return {
   setters: [],
   execute: function () {
     _export('default', exports.default);
   }
 };
});
