'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _PageHandler = require('./PageHandler');

var _PageHandler2 = _interopRequireDefault(_PageHandler);

var _Window = require('../../window/Window');

var _Window2 = _interopRequireDefault(_Window);

var _ClientRouter = require('../../router/ClientRouter');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 *
 */
class PageNavigationHandler extends _PageHandler2.default {
  static get $dependencies() {
    return [_Window2.default];
  }

  /**
   * @param {Window} window The utility for manipulating the global context
   *        and global client-side-specific APIs.
   */
  constructor(window) {
    super();

    /**
     * The utility for manipulating the global context and global
     * client-side-specific APIs.
     *
     * @type {ima.window.Window}
     */
    this._window = window;
  }

  /**
   * @inheritDoc
   */
  init() {
    // Setup history object to leave the scrolling to us and to not interfere
    const browserWindow = this._window.getWindow();

    if ('scrollRestoration' in browserWindow.history) {
      browserWindow.history.scrollRestoration = 'manual';
    }
  }

  /**
   * @inheritDoc
   */
  handlePreManagedState(managedPage, nextManagedState, action) {
    if (managedPage && action && action.type !== _ClientRouter.ActionTypes.POP_STATE) {
      this._saveScrollHistory();
      this._setAddressBar(action.url);
    }
  }

  /**
   * @inheritDoc
   */
  handlePostManagedState(managedPage, previousManagedPage, action) {
    const { event } = action;

    if (!event || !event.state || !event.state.scroll) {
      this._scrollTo({ x: 0, y: 0 });

      return;
    }

    this._scrollTo(event.state.scroll);
  }

  /**
   * Save user's scroll state to history.
   *
   * Replace scroll values in current state for actual scroll values in
   * document.
   */
  _saveScrollHistory() {
    const url = this._window.getUrl();
    const scroll = {
      x: this._window.getScrollX(),
      y: this._window.getScrollY()
    };
    const state = { url, scroll };

    const oldState = this._window.getHistoryState();
    const newState = Object.assign({}, oldState, state);

    this._window.replaceState(newState, null, url);
  }

  /**
   * Scrolls to give coordinates on a page.
   *
   * @param {Object} scroll
   * @param {number} [scroll.x]
   * @param {number} [scroll.y]
   */
  _scrollTo({ x = 0, y = 0 }) {
    setTimeout(() => {
      this._window.scrollTo(x, y);
    }, 0);
  }

  /**
   * Sets the provided URL to the browser's address bar by pushing a new
   * state to the history.
   *
   * The state object pushed to the history will be an object with the
   * following structure: {@code {url: string}}. The {@code url} field will
   * be set to the provided URL.
   *
   * @param {string} url The URL.
   */
  _setAddressBar(url) {
    let scroll = {
      x: 0,
      y: 0
    };
    let state = { url, scroll };

    this._window.pushState(state, null, url);
  }
}
exports.default = PageNavigationHandler;

typeof $IMA !== 'undefined' && $IMA !== null && $IMA.Loader && $IMA.Loader.register('ima/page/handler/PageNavigationHandler', [], function (_export, _context) {
 'use strict';
 return {
   setters: [],
   execute: function () {
     _export('default', exports.default);
   }
 };
});
